@@ -1630,6 +1630,12 @@ iconv_string_keeping_offsets (cd, s, n, offtable, t, m)
   const char *inptr;
   char *outptr;
   size_t outsize;
+  /* Avoid glibc-2.1 bug.  */
+#if !defined _LIBICONV_VERSION && (__GLIBC__ - 0 == 2 && __GLIBC_MINOR__ - 0 <= 1)
+  const size_t extra = 1;
+#else
+  const size_t extra = 0;
+#endif
 
   for (i = 0; i < n; i++)
     offtable[i] = (size_t)(-1);
@@ -1637,7 +1643,7 @@ iconv_string_keeping_offsets (cd, s, n, offtable, t, m)
   s_end = s + n;
   inptr = s;
   outptr = t;
-  outsize = m;
+  outsize = m + extra;
   while (inptr < s_end)
     {
       size_t insize;
@@ -1664,7 +1670,7 @@ iconv_string_keeping_offsets (cd, s, n, offtable, t, m)
     abort ();
 #endif
   /* We should have produced exactly m output bytes.  */
-  if (outsize != 0)
+  if (outsize != extra)
     abort ();
 }
 
@@ -1705,7 +1711,14 @@ mbs_possible_linebreaks (s, n, encoding, p)
   else
     {
 #if HAVE_ICONV
-      iconv_t to_utf8 = iconv_open (UTF8_NAME, encoding);
+      iconv_t to_utf8;
+      /* Avoid glibc-2.1 bug with EUC-KR.  */
+# if (__GLIBC__ - 0 == 2 && __GLIBC_MINOR__ - 0 <= 1) && !defined _LIBICONV_VERSION
+      if (STREQ (encoding, "EUC-KR", 'E', 'U', 'C', '-', 'K', 'R', 0, 0, 0))
+	to_utf8 = (iconv_t)(-1);
+      else
+# endif
+      to_utf8 = iconv_open (UTF8_NAME, encoding);
       if (to_utf8 != (iconv_t)(-1))
         {
           /* Determine the length of the resulting UTF-8 string.  */
@@ -1784,7 +1797,14 @@ mbs_width_linebreaks (s, n, width, start_column, at_end_columns, o, encoding, p)
   else
     {
 #if HAVE_ICONV
-      iconv_t to_utf8 = iconv_open (UTF8_NAME, encoding);
+      iconv_t to_utf8;
+      /* Avoid glibc-2.1 bug with EUC-KR.  */
+# if (__GLIBC__ - 0 == 2 && __GLIBC_MINOR__ - 0 <= 1) && !defined _LIBICONV_VERSION
+      if (STREQ (encoding, "EUC-KR", 'E', 'U', 'C', '-', 'K', 'R', 0, 0, 0))
+	to_utf8 = (iconv_t)(-1);
+      else
+# endif
+      to_utf8 = iconv_open (UTF8_NAME, encoding);
       if (to_utf8 != (iconv_t)(-1))
         {
           /* Determine the length of the resulting UTF-8 string.  */
